package models

import (
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	ct "gogs.mikescher.com/BlackForestBytes/goext/cursortoken"
	"regexp"
	"time"
)

type PostFilter struct {
	Type                 *PostType
	Organization         *OrganizationID
	Search               *string
	CreationTimeBefore   *time.Time
	CreationTimeAfter    *time.Time
	CreatorRef           *AnyID
	CreatorType          *UserType
	FrontendEnabled      *bool
	AutoEnableTimeBefore *time.Time
}

func (f PostFilter) FilterQuery() mongo.Pipeline {
	pipeline := mongo.Pipeline{}

	pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"deleted": nil}}})

	if f.Type != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"type": *f.Type}}})
	}

	if f.Organization != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"organizationID": *f.Organization}}})
	}

	if f.CreationTimeBefore != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"creationTime": bson.M{"$lt": *f.CreationTimeBefore}}}})
	}

	if f.CreationTimeAfter != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"creationTime": bson.M{"$gt": *f.CreationTimeAfter}}}})
	}

	if f.CreatorRef != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"creatorRef": *f.CreatorRef}}})
	}

	if f.CreatorType != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"creatorType": *f.CreatorType}}})
	}

	if f.FrontendEnabled != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"frontendEnabled": *f.FrontendEnabled}}})
	}

	if f.AutoEnableTimeBefore != nil {
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{"autoEnableTime": bson.M{"$ne": nil, "$lt": *f.AutoEnableTimeBefore}}}})
	}

	if f.Search != nil {
		ss := ".*" + regexp.QuoteMeta(*f.Search) + ".*"
		pipeline = append(pipeline, bson.D{{Key: "$match", Value: bson.M{
			"$or": bson.A{
				bson.M{"title": bson.M{"$regex": ss, "$options": "i"}},
				bson.M{"text": bson.M{"$regex": ss, "$options": "i"}},
			},
		}}})
	}

	return pipeline
}

func (f PostFilter) Pagination() (string, ct.SortDirection, string, ct.SortDirection) {
	return "creationTime", ct.SortDESC, "_id", ct.SortDESC
}
